-- @author        ClubPetey
-- Copyright (C) 2020, All Rights Reserved.

PriceFixing = {};
PriceFixing.dir = getUserProfileAppPath() .. "mods/";
PriceFixing.version = "1.0.0.0";
PriceFixing.versionDate = "20.10.2020";
PriceFixing.currentVersionId = 1000; 
PriceFixing.configFileName = "PriceFixing.xml";
PriceFixing.initialized = false;
PriceFixing.fillType = {}

function init() 
  addConsoleCommand("priceList", "Send a price list of store items to log.txt", "priceList", PriceFixing);
  addModEventListener(PriceFixing);

  FSBaseMission.loadMapFinished = Utils.appendedFunction(FSBaseMission.loadMapFinished, mapLoaded)
end

function mapLoaded()
  PriceFixing:onMapLoaded();
end

-- In theory at this point the economy hasn't been activated, but the types have been loaded.
-- Changes should be done here. But I grab the original prices for the priceList function
function PriceFixing:onMapLoaded()
  i = 0;  
  while true do
    local keySub = string.format("PriceFixing.change_filltype(%d)", i);  
    if not hasXMLProperty(self.xmlFile, keySub) then
      break;
    end;

    local price = Utils.getNoNil(getXMLFloat(self.xmlFile, keySub.."#price"), -1.0);
    local name = Utils.getNoNil(getXMLString(self.xmlFile, keySub.."#id"), "");   

    if name ~= "" and price ~= -1 then
      ft = g_fillTypeManager:getFillTypeByName(name:upper());
      if ft ~= nil then
        self.log:debug(string.format("FillType change: [%s] %f -> %f", name, ft.pricePerLiter, price));
        ft.pricePerLiter = price;
      end
    end

    i = i + 1;
  end

  types = g_fillTypeManager:getFillTypes()
  for _, ft in pairs(types) do    
    ftdata = {}
    ftdata.pricePerLiter = ft.pricePerLiter;
    ftdata.name = ft.name;
    self.fillType[ft.name] = ftdata;
  end

  delete(self.xmlFile);
end

function PriceFixing:expandItemTag(tag) 
  tag = tag:lower()
  if (tag:sub(1, 1) == "~") then
    return PriceFixing.dir .. tag:sub(3)
  else 
    return tag
  end
end

function PriceFixing:createDefaultConfig(fileName)
  createFolder(getUserProfileAppPath() .. "modsSettings/")
  local xml = createXMLFile("PriceFixing", fileName, "PriceFixing");
  setXMLString(xml, "PriceFixing.change#file", "~/example_mod/mod_item.xml")
  setXMLInt(xml, "PriceFixing.change#price", 1000000)
  saveXMLFile(xml)
  delete(xml)
end

function PriceFixing:priceList() 
  local lvl = self.log:getLevel();

  self.log:debug("--- START PRICE LIST ---");
  local items = g_storeManager:getItems();

  for _, item in pairs(items) do
    if (item.canBeSold and item.showInStore) then
      self.log:debug(item.xmlFilename .. " (" .. item.name .. ") = " .. item.price .. " / " .. item.dailyUpkeep);
    else 
      self.log:debug(item.xmlFilename .. " (**" .. item.name .. "**) = " .. item.price);
    end

    --if item.configurations ~= nil then
    --  for k, v in pairs(item.configurations) do
    --    for _, c in ipairs(v) do
    --      if c.name ~= nil and c.name ~= "" then
    --        self.log:debug("-- " .. k .. "-" .. c.index .. "(" .. c.name .. ") = " .. c.price);
    --      end
    --    end
    --  end    
    --end
    
  end
  self.log:debug("--- END PRICE LIST ---");
  self.log:debug("--- START TYPE LIST ---");
  for _, v in pairs(self.fillType) do    
    self.log:debug(string.format("%s = %f", v.name, v.pricePerLiter));
  end
  self.log:debug("--- END TYPE LIST ---");

  self.log:setLevel(lvl);
end

function PriceFixing:loadMap(name)
  self.log = CPLog:create("PriceFixing");
  self.log:setLevel(CPLog.DEBUG);

  local configFile = getUserProfileAppPath() .. "modsSettings/" .. PriceFixing.configFileName;

  self.log:debug("Mod directory is: " .. PriceFixing.dir);
  self.log:info("Looking for price changes in: " .. configFile);
  
  if not fileExists(configFile) then
    self.log:info("No change file found -- creating a new one");
    self:createDefaultConfig(configFile)
    return
  end

  xmlFile = loadXMLFile("PriceFixing", configFile);
  self.xmlFile = xmlFile

  i = 0;
  skip = 0;

  while true do
    local keySub = string.format("PriceFixing.change(%d)", i);
    
    if not hasXMLProperty(xmlFile, keySub) then
      break;
    end;

    local itemTag = getXMLString(xmlFile, keySub.."#file");
    

    local price = Utils.getNoNil(getXMLInt(xmlFile, keySub.."#price"), -1);
    if (price == -1) then 
      price = Utils.getNoNil(getXMLInt(xmlFile, keySub..".price"), -1);
    end

    local hide = Utils.getNoNil(getXMLBool(xmlFile, keySub.."#hide"), false);
    local name = Utils.getNoNil(getXMLString(xmlFile, keySub..".name"), "");
    local upkeep = Utils.getNoNil(getXMLString(xmlFile, keySub..".upkeep"), -1);
    
    if itemTag == nil then 
      self.log:warn(string.format("Skipping change %d: No itemTag", i));
      skip = skip + 1;
    else
      itemTag = self:expandItemTag(itemTag);
      local item = g_storeManager:getItemByXMLFilename(itemTag);

      if item == nil then
        self.log:warn(string.format("Skipping change %d: Item not found", i));
        skip = skip + 1;
      else
        if (hide) then
          item.showInStore = false
          self.log:debug(string.format("Removed %d: %s", i, itemTag));
        else
          if (price ~= -1) then
            self.log:debug(string.format("Change price %d: %s -> %s", i, item.price, price));
            item.price = price;
          end

          if (name ~= "") then
            self.log:debug(string.format("Change name %d: %s -> %s", i, item.name, name));
            item.name = name;
          end

          if (upkeep ~= -1) then
            self.log:debug(string.format("Change upkeep %d: %s -> %s", i, item.dailyUpkeep , upkeep));
            item.dailyUpkeep  = upkeep;
          end
        end
      end      
    end
    i = i + 1;
  end;  

  self.log:info(string.format("Made %d item changes", i - skip));

  i = 0;

  local globalConfigChanges = {}  

  while true do
    local keySub = string.format("PriceFixing.change_config(%d)", i);  
    if not hasXMLProperty(xmlFile, keySub) then
      break;
    end;

    local change = {};
    change.price = Utils.getNoNil(getXMLInt(xmlFile, keySub.."#price"), -1);    
    local name = Utils.getNoNil(getXMLString(xmlFile, keySub.."#id"), "");    
    if name ~= "" then
      local index = string.find(name, "-[^-]*$");
      change.id = name:sub(1, index-1);
      change.idx = tonumber(name:sub(index+1));

      self.log:debug(string.format("Global Price change: %s[%d] -> %d", change.id, change.idx, change.price));
      table.insert(globalConfigChanges, change)
    end
    i = i + 1;
  end

  if (table.getn(globalConfigChanges) > 0) then
    i = 0;
    local items = g_storeManager:getItems();
    for _, item in pairs(items) do
      for _, c in ipairs(globalConfigChanges) do
        if (item.configurations ~= nil and item.configurations[c.id] ~= nil and item.configurations[c.id][c.idx] ~= nil) then
          item.configurations[c.id][c.idx].price = c.price;
          i = i + 1;
        end      
      end
    end
    self.log:info(string.format("Made %d global config changes", i));
  end

  -- this will be done in onMapLoaded()
  -- delete(xmlFile)
end

function PriceFixing:deleteMap()
end

function PriceFixing:mouseEvent(posX, posY, isDown, isUp, button)
end

function PriceFixing:keyEvent(unicode, sym, modifier, isDown)
end

function PriceFixing:update(dt)
end

function PriceFixing:draw()
  if g_dedicatedServerInfo ~= nil or not g_currentMission.renderTime or not g_currentMission.hud.showTime then
    return
  end

end

init()

